<?php
/**
 * @version		$Id: passwordstrength.php 20196 2011-03-04 02:40:25Z mrichey $
 * @package		plg_auth_email
 * @copyright	Copyright (C) 2005 - 2011 Michael Richey. All rights reserved.
 * @license		GNU General Public License version 2 or later; see LICENSE.txt
 */

// No direct access
defined('_JEXEC') or die;

jimport('joomla.plugin.plugin');

class plgUserPasswordStrength extends JPlugin
 {

    /**
     * @param	JForm	$form	The form to be altered.
     * @param	array	$data	The associated data for the form.
     *
     * @return	boolean
     * @since	1.6
     */
    function onContentPrepareForm($form, $data) {
        if (!($form instanceof JForm)) {
            $this->_subject->setError('JERROR_NOT_A_FORM');
            return false;
        }
        
        $app = JFactory::getApplication();
        $validforms=array(
            'com_admin.profile'=>array(
                'inputname'=>'jform[password1]',
                'inputid'=>'password'
            ),
            'com_users.profile'=>array(
                'inputname'=>'jform[password1]',
                'inputid'=>'password'
            ),
            'com_users.registration'=>array(
                'inputname'=>'jform[password1]',
                'inputid'=>'password'
            ),
        );
        if($app->isAdmin()) {
            $validforms['com_users.user']=array(
                'inputname'=>'jform[password]',
                'inputid'=>'password'
            );
            $validforms['com_admin.profile']['inputname']='jform[password]';
            $adminscript=array(
                "window.addEvent('domready',function(){",
                "\tdocument.id('containerId').setStyles({",
                "\t\t'float':'left',",
                "\t\t'margin-top':'5px'",
                "\t});",
                "});"
            );
            $adminscript=implode("\n",$adminscript);
        } else {
            $validforms['com_users.user']='jform[password1]';
            $adminscript='';
        }
        $customforms=$this->params->get('customforms','');
        if(strlen($customforms)) {
            $customforms=array();
            foreach(explode("\n",$this->params->get('customforms','')) as $cf) {
                $cf = explode(":",$cf);
                $customforms[$cf[0]]=array('inputname'=>$cf[2],'inputid'=>$cf[1]);
            }
            $validforms = array_merge($validforms,$customforms);
        }
        // Check we are manipulating a valid form.
        $formname = $form->getName();
        if (!in_array($formname, array_keys($validforms))) return true;

        if($this->chooseSystem()) {
            $doc = JFactory::getDocument();
            JFactory::getLanguage()->load('plg_user_passwordstrength',JPATH_ADMINISTRATOR);
            JHtml::_('behavior.framework',true);
            $doc->addScript(JURI::root(true).DIRECTORY_SEPARATOR.'media'.DIRECTORY_SEPARATOR.'plg_user_passwordstrength'.DIRECTORY_SEPARATOR.'js'.DIRECTORY_SEPARATOR.'password-strength-meter.js');
            $doc->addScriptDeclaration($this->getJs($validforms[$formname]['inputname'])."\n".$adminscript);
            $doc->addStyleDeclaration($this->getCss($app->isAdmin()));
            return true;
        } else {
            $form->setFieldAttribute($validforms[$formname]['inputid'],'strengthmeter','true');
            $form->setFieldAttribute($validforms[$formname]['inputid'],'threshold',$this->params->get('threshold',33));
        }
    }
    private function chooseSystem(){
        if(!$this->params->get('builtinmeter',0)) {
            return true;
        }
        $version = explode('.',JVERSION);
        if($version[0] < 2) {
            return true;
        }
        return false;
    }
    private function getJs($inputname){
        $script[]="window.addEvent('domready',function(){";
        $script[]="\tvar inputelement = $(document.body).getElement('[name=".$inputname."]');";
        $script[]="\tif(typeOf(inputelement)!='null') {";
        $script[]="\t\tvar pass = new PassMeter('".$inputname."',{";
        $script[]=$this->getJsOptions();
        $script[]="\t\t});";
        $script[]="\t};";
        $script[]="});";
        return implode("\n",$script);
    }
    private function getJsOptions() {
        $options=array();
        $options[]="\t\t\t'minLength':".$this->params->get('minLength',6);
        $options[]="\t\t\t'maxLength':".$this->params->get('maxLength',15);
        if($this->params->get('useFx',1)) {
            $options[]="\t\t\t'useFx':true";
            $transition=$this->params->get('transitionmethod','Sine');
            $transition.=(($this->params->get('transitionmethod','Sine')!='linear')?'.'.$this->params->get('transition','easeOut'):'');
            $options[]="\t\t\t'transition':Fx.Transitions.".$transition;
            $options[]="\t\t\t'duration':".$this->params->get('duration',250);
        } else {
            $options[]="\t\t\t'useFx':false";
        }
        if($this->params->get('usetext',1)) {
            $options[]="\t\t\t'useText':true";
            $options[]="\t\t\t'poorText':'".JText::_('PLG_USER_PASSWORDSTRENGTH_TEXT_POOR')."'";
            $options[]="\t\t\t'weakText':'".JText::_('PLG_USER_PASSWORDSTRENGTH_TEXT_WEAK')."'";
            $options[]="\t\t\t'mediumText':'".JText::_('PLG_USER_PASSWORDSTRENGTH_TEXT_MEDIUM')."'";
            $options[]="\t\t\t'strongText':'".JText::_('PLG_USER_PASSWORDSTRENGTH_TEXT_STRONG')."'";
        } else {
            $options[]="\t\t\t'useText':false";
        }
        if($this->params->get('usedefaultstrength',1)) {
            $options[]="\t\t\t'weakRegEx':'([a-z]{1}[A-Z]{1})|([A-Z]{1}[a-z]{1})[^\s]'";
            $options[]="\t\t\t'mediumRegEx':'[0-9]{1}'";
            $options[]="\t\t\t'strongRegEx':'[^a-zA-Z0-9]{1}'";
        } else {
            $options[]="\t\t\t'weakRegEx':'".$this->params->get('weakregex','([a-z]{1}[A-Z]{1})|([A-Z]{1}[a-z]{1})[^\s]')."'";
            $options[]="\t\t\t'mediumRegEx':'".$this->params->get('mediumregex','[0-9]{1}')."'";
            $options[]="\t\t\t'strongRegEx':'".$this->params->get('strongregex','[^a-zA-Z0-9]{1}')."'";
        }
        return implode(",\n",$options);
    }

    private function getCss($admin) {
        /* meter outer container */
        if($this->params->get('usecustomcss',0) == 0 && !$admin) {
            $css=array();
            $css[]='#containerId, #meterId {display:block;}';
            $css[]='#containerId {width:200px; border:1px solid #000; background:#000; color: #fff; margin:0; padding:0px;}';
            $css[]='.poorPass {width:48px; border:1px solid #f00; background:#f00;text-indent:10px;}';
            $css[]='.weakPass {width:73px; border:1px solid #ff0; background: #ff0;text-indent:10px; color:#000;}';
            $css[]='.mediumPass {width:148px; border:1px solid #00f; background:#00f;text-indent:10px; color:#fff;}';
            $css[]='.strongPass {width:198px; border:1px solid #0f0; background: #0f0;text-indent:10px; color:#000;}';
            return implode("\n",$css);
        } else {
            return $this->params->get('customcss','');
        }
    }
}
